﻿using System;
using UnityEngine;
using Windows.Kinect;

public class CoordinateMapperView : MonoBehaviour
{
	private ComputeBuffer DepthBuffer,
                          BodyIndexBuffer;

	private DepthSpacePoint[] DepthPoints;
	private byte[] BodyIndexPoints;
	
	void Start ()
	{
		Release();

        CoordinateMapperManager CoordinateMapperManager = GameObject.Find("Managers").GetComponent<CoordinateMapperManager>();
		
        GetComponent<Renderer>().material.SetTexture("_MainTex", CoordinateMapperManager.GetColorTexture());

        Func<Array, Array, Action<string, ComputeBuffer, int>> setBuffer = (srcPoints, dstPoints) =>
        {
            dstPoints = srcPoints;
            return (name, buffer, stride) =>
            {
                buffer = new ComputeBuffer(dstPoints.Length, stride);
                GetComponent<Renderer>().material.SetBuffer(name, buffer);
            };
        };

        setBuffer(CoordinateMapperManager.GetDepthCoordinates(), DepthPoints)
            ("depthCoordinates", DepthBuffer, sizeof(float) * 2);

        setBuffer(CoordinateMapperManager.GetBodyIndexBuffer(), BodyIndexPoints)
            ("BodyIndexBuffer", BodyIndexBuffer, sizeof(float));
    }

	void Update()
	{
		//TODO: fix perf on this call.
		DepthBuffer.SetData(DepthPoints);
        
		BodyIndexBuffer.SetData(System.Array.ConvertAll(BodyIndexPoints, System.Convert.ToSingle));
	}
	
	private void Release() 
	{
		if (DepthBuffer != null)
			DepthBuffer.Release();

		if (BodyIndexBuffer != null)
			BodyIndexBuffer.Release();

		DepthPoints = null;
		BodyIndexPoints = null;
	}
	
	void OnDisable() 
	{
		Release();
	}
}
